summaryrefslogtreecommitdiff
path: root/support/asymptote/__main__.py
diff options
context:
space:
mode:
authorMichael Schwarz <michi.schwarz@gmail.com>2015-09-14 15:04:45 +0200
committerMichael Schwarz <michi.schwarz@gmail.com>2015-09-16 01:50:24 +0200
commit24a60718eb64da9f5b9e284965e888dd4371a278 (patch)
treed6ec06a0890df90361e0f4be28ad246052f57213 /support/asymptote/__main__.py
parenta1f09b7f86c0d20941562f5b1254f9c0a2b9a535 (diff)
downloadpogojig-24a60718eb64da9f5b9e284965e888dd4371a278.tar.gz
pogojig-24a60718eb64da9f5b9e284965e888dd4371a278.tar.bz2
pogojig-24a60718eb64da9f5b9e284965e888dd4371a278.zip
Implemented Asymptote dependency tracking.
Diffstat (limited to 'support/asymptote/__main__.py')
-rw-r--r--support/asymptote/__main__.py62
1 files changed, 45 insertions, 17 deletions
diff --git a/support/asymptote/__main__.py b/support/asymptote/__main__.py
index 238332f..9d712ef 100644
--- a/support/asymptote/__main__.py
+++ b/support/asymptote/__main__.py
@@ -1,26 +1,54 @@
-import os, shutil
-from lib import util
+import sys, os, shutil
+from lib import util, make
def _asymptote(in_path, out_path, asymptote_dir, working_dir):
- util.command([os.environ['ASYMPTOTE'], '-f', 'pdf', '-o', out_path, in_path], set_env = { 'ASYMPTOTE_DIR': asymptote_dir }, working_dir = working_dir)
+ args = [os.environ['ASYMPTOTE'], '-vv', '-f', 'pdf', '-o', out_path, in_path]
+
+ with util.command_context(args, set_env = { 'ASYMPTOTE_DIR': asymptote_dir }, working_dir = working_dir, use_stderr = True) as process:
+ def get_loaded_file(line):
+ if any(line.startswith(j) for j in ['Loading ', 'Including ']):
+ parts = line.rstrip('\n').split(' ')
+
+ if len(parts) == 4:
+ _, _, from_, path = parts
+
+ if from_ == 'from':
+ return path
+
+ return None
+
+ def iter_loaded_files():
+ for i in process.stderr:
+ loaded_file = get_loaded_file(i)
+
+ if loaded_file is not None:
+ yield loaded_file
+ elif not any(i.startswith(j) for j in ['cd ', 'Using configuration ']):
+ print >> sys.stderr, i,
+
+ loaded_files = list(iter_loaded_files())
+
+ return loaded_files
@util.main
def main(in_path, out_path):
_, out_suffix = os.path.splitext(out_path)
- if out_suffix == '.pdf':
- with util.TemporaryDirectory() as temp_dir:
- absolute_in_path = os.path.abspath(in_path)
- temp_out_path = os.path.join(temp_dir, 'out.pdf')
-
- # Asymptote creates A LOT of temp files (presumably when invoking LaTeX) and leaves some of them behind. Thus we run asymptote in a temporary directory.
- _asymptote(absolute_in_path, 'out', os.path.dirname(absolute_in_path), temp_dir)
-
- if not os.path.exists(temp_out_path):
- raise util.UserError('Asymptote did not generate a PDF file for input file {}.', in_path)
-
- shutil.copyfile(temp_out_path, out_path)
- else:
- raise Exception('Unknown file type: {}'.format(out_suffix))
+ with util.TemporaryDirectory() as temp_dir:
+ absolute_in_path = os.path.abspath(in_path)
+ temp_out_path = os.path.join(temp_dir, 'out.pdf')
+
+ # Asymptote creates A LOT of temp files (presumably when invoking LaTeX) and leaves some of them behind. Thus we run asymptote in a temporary directory.
+ loaded_files = _asymptote(absolute_in_path, 'out', os.path.dirname(absolute_in_path), temp_dir)
+
+ if not os.path.exists(temp_out_path):
+ raise util.UserError('Asymptote did not generate a PDF file for input file {}.', in_path)
+
+ # All dependencies as paths relative to the project root.
+ dependencies = set(map(os.path.relpath, loaded_files))
+
+ # Write output files.
+ make.write_dependencies(out_path + '.d', out_path, dependencies - { in_path })
+ shutil.copyfile(temp_out_path, out_path)